require 'spec_helper'

describe DumpTruck::Mysql::Translator do
  let(:translator){DumpTruck::Mysql::Translator.new}

  describe '#table?' do
    context "when line contains table create" do
      let(:line){"CREATE TABLE `roles` (\n"}

      it 'returns truthy' do
        expect(translator).to be_table(line)
      end
    end

    context "when line doesn't contain table create" do
      let(:line){") ENGINE=InnoDB AUTO_INCREMENT=94 DEFAULT CHARSET=utf8;\n"}

      it 'returns falsey' do
        expect(translator).to_not be_table(line)
      end
    end
  end

  describe '#extract_table' do
    context "when line contains table create" do
      let(:line){"CREATE TABLE `roles` (\n"}

      it 'extracts the table name' do
        expect(translator.extract_table(line)).to eq('roles')
      end
    end

    context "when line doesn't contain table create" do
      let(:line){") ENGINE=InnoDB AUTO_INCREMENT=94 DEFAULT CHARSET=utf8;\n"}

      it 'returns nil' do
        expect(translator.extract_table(line)).to be_nil
      end
    end
  end

  describe '#insert?' do
    context "when line contains insert" do
      let(:line){"INSERT INTO `roles` (`id`, `name`, `description`, `created_at`, `updated_at`) VALUES (1,'Employee',NULL,'2009-08-11 11:10:17','2009-08-11 11:10:17');\n"}

      it 'returns truthy' do
        expect(translator).to be_insert(line)
      end
    end

    context "when line doesn't contain insert" do
      let(:line){") ENGINE=InnoDB AUTO_INCREMENT=94 DEFAULT CHARSET=utf8;\n"}

      it 'returns falsey' do
        expect(translator).to_not be_insert(line)
      end
    end
  end

  describe '#extract_insert' do
    context 'with typical example' do
      let(:line){"INSERT INTO `roles` (`id`, `name`, `description`, `created_at`, `updated_at`) VALUES (1,'Employee',NULL,'2009-08-11 11:10:17','2009-08-11 11:10:17');\n"}

      it 'extracts the fields and values' do
        expect(translator.extract_insert(line)).to eq([
          %w(id name description created_at updated_at),
          [{
            'id' => 1,
            'name' => 'Employee',
            'description' => nil,
            'created_at' => '2009-08-11 11:10:17',
            'updated_at' => '2009-08-11 11:10:17'
          }]
        ])
      end
    end

    context 'with special cases' do
      let(:line){"INSERT INTO `special_cases` (`contains_escaped_quote`, `empty_string`, `escapes_slash_before_quote`, `negative_number`, `negative_float`) VALUES ('\\'','','\\\\',-1,-1.3);\n"}

      it 'extracts the fields and values' do
        expect(translator.extract_insert(line)).to eq([
          %w(contains_escaped_quote empty_string escapes_slash_before_quote negative_number negative_float),
          [{
            'contains_escaped_quote' => "\\'",
            'empty_string' => '',
            'escapes_slash_before_quote' => "\\\\",
            'negative_number' => -1,
            'negative_float' => -1.3
          }]
        ])
      end
    end
  end

  describe '#generate_insert' do
    let(:table){'roles'}
    let(:keys){%w(id name description created_at updated_at)}
    let(:data) do
      [{
        'id' => 1,
        'name' => 'Employee',
        'description' => nil,
        'created_at' => '2009-08-11 11:10:17',
        'updated_at' => '2009-08-11 11:10:17'
      }]
    end

    it 'generates a valid insert line' do
      expect(translator.generate_insert(table, keys, data)).to eq(
        "INSERT INTO `roles` (`id`, `name`, `description`, `created_at`, `updated_at`) VALUES (1,'Employee',NULL,'2009-08-11 11:10:17','2009-08-11 11:10:17');\n"
      )
    end
  end
end
